﻿using System;
using System.Text;
using System.IO;
using System.Speech;
using System.Windows;
using Microsoft.Kinect;
using Microsoft.Speech.AudioFormat;
using Microsoft.Speech.Recognition;
using System.ComponentModel;

namespace KinectForWindowsSpeech
{

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private KinectSensor kinect;

        private SpeechRecognitionEngine spRecEng;

        private const double ConfidenceThreshold = 0.8;


        public MainWindow()
        {
            InitializeComponent();
            WindowLoaded();
        }
        ~MainWindow()
        {
            WindowClosing();
        }

        private static RecognizerInfo GetKinectRecognizer()
        {
            foreach (RecognizerInfo recgonizer in SpeechRecognitionEngine.InstalledRecognizers())
            {
                string value;
                recgonizer.AdditionalInfo.TryGetValue("Kinect", out value);
                if ("True".Equals(value, StringComparison.OrdinalIgnoreCase) && "en-US".Equals(recgonizer.Culture.Name, StringComparison.OrdinalIgnoreCase))
                {
                    return recgonizer;
                }
            }
            return null;
        }

        private void WindowLoaded()
        {
            foreach (var potentialSensor in KinectSensor.KinectSensors)
            {
                if (potentialSensor.Status == KinectStatus.Connected)
                {
                    this.kinect = potentialSensor;
                    break;
                }
            }
            if (this.kinect != null)
            {
                try
                {
                    this.kinect.Start();
                }
                catch (IOException)
                {
                    // we can get this error if another application in streaming from the same Kinect
                    this.kinect = null;
                }
            }
            //todo: show there is no Kinect available

            RecognizerInfo ri = GetKinectRecognizer();
            if (ri != null)
            {
                this.spRecEng = new SpeechRecognitionEngine(ri.Id);
                using (var memoryStream = new MemoryStream(Encoding.ASCII.GetBytes(Properties.Resources.SpeechGrammar)))
                {
                    var g = new Grammar(memoryStream);
                    spRecEng.LoadGrammar(g);
                }
                spRecEng.SpeechRecognized += spRecEng_SpeechRecognized;
                spRecEng.SpeechRecognitionRejected += spRecEng_SpeechRecognitionRejected;

                spRecEng.SetInputToAudioStream(kinect.AudioSource.Start(), new SpeechAudioFormatInfo(EncodingFormat.Pcm, 16000, 16, 1, 32000, 2, null));
                spRecEng.RecognizeAsync(RecognizeMode.Multiple);
            }
            else
            {
                //todo: deal with no speech recognizer
            }
        }

        private void WindowClosing()
        {
            if (this.kinect != null)
            {
                this.kinect.AudioSource.Stop();
                this.kinect = null;
            }
            if (this.spRecEng != null)
            {
                this.spRecEng.SpeechRecognized -= spRecEng_SpeechRecognized;
                this.spRecEng.SpeechRecognitionRejected -= spRecEng_SpeechRecognitionRejected;
                this.spRecEng.RecognizeAsyncStop();
            }
        }

        void spRecEng_SpeechRecognitionRejected(object sender, SpeechRecognitionRejectedEventArgs e)
        {
            this.semanticRep.Text="What did you say?";
            this.spoken.Text = "???";
        }

        void spRecEng_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            if (e.Result.Confidence < ConfidenceThreshold)
            {
                //engine is not confident about result => ignore
                return;
            }
            string semantic = "";
            switch (e.Result.Semantics.Value.ToString())
            {
                case "PEOPLE":
                    semantic = "You are talking about people";
                    break;
                case "TOYS":
                    semantic = "You are talking about toys";
                    break;
                case "FOOD":
                    semantic = "You are talking about food";
                    break;
                case "COMMANDS":
                    this.Close();
                    break;
                default:
                    semantic = "I am confident that I heard something, but I don't know what";
                    break;
            }
            semanticRep.Text = semantic;
            spoken.Text = e.Result.Text;
        }
        
    }
}
